// Interactive Web-Based Pointillist Visualization of Hydrogenic Orbitals Using Jmol
//
// Shane P. Tully, Thomas M. Stitt, Robert D. Caldwell, Brian J. Hardock, 
// Robert M. Hanson, and Przemyslaw Maslak
//
// Department of Chemistry, the Pennsylvania State University, University
// Park, PA 16802, and Department of Chemistry, St. Olaf College, Northfield,
// MN 55057

var radii;                                      // lengths from (0, 0, 0)
var jmolFlotData;                               // the data from the jmol plot for flot
var mcSeed;
var mcN;
var mcL;
var mcPoints;
var mcStep;
var mcMostProbRadius;
var mcAvgRadius;


function getMCN()
{
    return mcN;
}

function getMCL()
{
    return mcL;
}

function montecarloGetMostProbablyRadius()
{
    return mcMostProbRadius;
}

function montecarloGetAverageRadius()
{
    return mcAvgRadius;
}

function getJmolData(n, l, seed, numPoints, vertices, stepSize)
{
    if ( !(n == mcN && l == mcL && seed == mcSeed && numPoints == mcPoints && stepSize == mcStep) )
    {
        calculateTheoPlots(n, l, stepSize);
        calcJmolData(vertices, stepSize);
        mcN = n;
        mcL = l;
        mcSeed = seed;
        mcPoints = numPoints;
    }
    return jmolFlotData;
}

function getMCStep()
{
    return mcStep;
}

function getPartitionedJmolData(xCutoff)
// PRE:  calcJmolData must have been called.  Goes from 0 to xCutoff
{
    var end = Math.floor(xCutoff/mcStep);
    
    if ( end > jmolFlotData.length )
        end = jmolFlotData.length;
    
    return jmolFlotData.slice(0, end);
}



function calcJmolData(vertices, stepSize)
// PRE:  vertices must be defined (taken from Jmol if anything useful is desired). stepSize > 0 if desired (blank if auto)
// POST: FCTVAL = x data (radii) and corresponding probability (normailized y) 
{
    var index = 0;                        // The index in vrmlData
    var maxRadius;                        // max radius
    var mprValue;                         // most probably radius value
    var totalVals;
    var numVal;
    var theoMPR;                          // the theoretical most probable radius
    var mcMPRRange;                       // the range (in stepSize size steps) to get the most prob radius
    var mcMPRIndex;
        
    
    mprValue = 0;
    numVal = 0;
    totalVals = 0;
    arrayIndex = 0;
    radiusIndex = 0;
    radii = new Array();
        
    jmolFlotData = new Array();
    for (var i = 0; i < vertices.length; i++)                               // find length from center of each vertex and add to radii array
    {
        radii[i] = Math.sqrt( Math.pow(vertices[i][0], 2) + Math.pow(vertices[i][1], 2) + Math.pow(vertices[i][2], 2) );

        if (i == 0)                                                         // find the maximum radius
            maxRadius = radii[i];
        else if (radii[i] > maxRadius)
            maxRadius = radii[i];
    }

    //vrmlStart = radii.length - 1;

    if (stepSize == null)
    {
        stepSize = Math.ceil(maxRadius) / 80;
        //stepSize = Math.round(maxRadius * 10) / 1000;                       // around 100 "categories" appears to produce nice thin bars so get the step size to three decimals
    }

    mcStep = stepSize;

    for (var i = 0; i < Math.ceil( maxRadius / stepSize ); i++ )            // populate jmolFlotData with x and y
    {
        jmolFlotData.push( [stepSize * i, 0]);
    }

    for (var i = 0; i < radii.length; i++)                                  // add to a "category" if the data is in the range
    {
        index = Math.floor( radii[i] / stepSize );
        jmolFlotData[index][1]++;
        
        if (jmolFlotData[index][1] > mprValue)
        {
            mprValue = jmolFlotData[index][1];
            mostProbableRadius = jmolFlotData[index][0];
        }
    }
    
    for (var i = 0; i < jmolFlotData.length; i++)
    {
        totalVals += jmolFlotData[i][1];
        numVal += jmolFlotData[i][0] * jmolFlotData[i][1];
    }
    
    mcAvgRadius = numVal / totalVals;
    
    // Calculate the MC most probable radius
    
    totalVals = 0;
    numVal = 0;
    
    
    // need a range for where the mpr will be and in that range take an average and find the most probable radius.
    //  a range of three is fine unless ceil(extent / stepSize) is small as then an indexOutOfBounds exception will occur
    //  when trying to find the "rolling average"
    
    // use three unless small.  Use 0 (ex just take middle bar) if step is large
    mcMPRRange = 3;
    theoMPR = schrodingerGetMostProbablyRadius();
    mcMPRIndex = Math.round(theoMPR / mcStep);
    
    // change mcMPRRange if too large
    while ( mcMPRRange > 0 && (mcMPRIndex - mcMPRRange < 0 ||  mcMPRIndex + mcMPRRange > jmolFlotData.length) )
    {
        mcMPRRange--;
    }
    
    for (var i = mcMPRIndex - mcMPRRange; i < mcMPRIndex + mcMPRRange; i++) // throw a check in
    {
        totalVals += jmolFlotData[i][1];
        numVal += jmolFlotData[i][0] * jmolFlotData[i][1];
    }
    
    mcMostProbRadius = numVal / totalVals;
    

    
    for (var i = 0; i < jmolFlotData.length; i++)                               //  JUST normalize the data |||| remove "useless" (under cutoff) values and
    {
        /*if (jmolFlotData[i][1] <= cutoff)
        {
            jmolFlotData.splice(i, 1);
            i--;
        }
        else*/
            jmolFlotData[i][1] = jmolFlotData[i][1] / mprValue;
    }
    
    
    return jmolFlotData;
}