// Interactive Web-Based Pointillist Visualization of Hydrogenic Orbitals Using Jmol
//
// Shane P. Tully, Thomas M. Stitt, Robert D. Caldwell, Brian J. Hardock, 
// Robert M. Hanson, and Przemyslaw Maslak
//
// Department of Chemistry, the Pennsylvania State University, University
// Park, PA 16802, and Department of Chemistry, St. Olaf College, Northfield,
// MN 55057

var CONST_STEP_SIZE = 0.01;                     // Stepsize that flot is using for R, R^2 and (r^2)*(R^2) plots
var DEFAULT_EXTENT = 10;                        // The default extent

var ORBITAL_EXTENTS = [[3], [10, 10], [20, 20, 20], [30, 30, 30, 25], [40, 40, 40, 40, 40]];
var ORBITAL_PROBABILITY_50 = [[1.34], [5.8, 4.67], [13.62, 12.57, 10.01], [24.9, 23.87, 21.6, 17.34], [39.7, 38.68, 36.5, 32.85, 26.67]];
var ORBITAL_PROBABILITY_90 = [[2.66], [9.13, 7.99], [19.44, 18.39, 15.8], [33.62, 32.59, 30.31, 25.99], [51.68, 50.66, 48.51, 44.82, 38.52]];
var ORBITAL_PROBABILITY_95 = [[3.15], [10.28, 9.15], [21.39, 20.24, 17.76], [36.47, 35.45, 33.17, 28.87], [55.52, 54.5, 52.34, 48.68, 42.4]];


var posr2p2Data;                            // positive psi^2 * 4pir^2 values
var negr2p2Data;                            // negative psi^2 * 4pir^2 values
var posp2Data;                              // positive psi^2 values
var negp2Data;                              // negative psi^2 values
var posPsiData;                             // pos psi values
var negPsiData;                             // neg psi values
var r2R2MarkerArray;                        // marker data [v1, v2, v3]
var rMarkerArray;                           // marker data [v1, v2, v3]
var r2MarkerArray;                          // marker data [v1, v2, v3]

var schrodingerMostProbRadius;
var schrodingerAvgRadius;
var schrodingerN;
var schrodingerL;
//var schrodingerStep;
var rSingleArray;


function schrodingerGetMostProbablyRadius()
{
    return schrodingerMostProbRadius;
}

function schroMPR(n, l)
{
    return 0;
}

function schrodingerGetAverageRadius()
{
    return schrodingerAvgRadius;
}

function getWaveFunction(n, l)
{
    calculateTheoPlots(n, l);     
    return[posPsiData, negPsiData];
}

function getElectronDensity(n, l)
{
    calculateTheoPlots(n, l);
    return[posp2Data, negp2Data];
}

function getRadialProbabilityDistribution(n, l)
{
    calculateTheoPlots(n, l);
    return[posr2p2Data, negr2p2Data];
}

function getWaveFunctionSingleArray(n, l)
{
    calculateTheoPlots(n, l);
    return rSingleArray;
}


function calculateTheoPlots(n, l)
{
    if ( !(schrodingerN == n && schrodingerL == l /*&& schrodingerStep == stepSize*/) )
    {
        schrodingerN = n;
        schrodingerL = l;
        //schrodingerStep = stepSize;
        getSchrodingerData(n, l);
    }
}


function getr2R2Marker50()
{
    return r2R2MarkerArray[0];
}

function getr2R2Marker90()
{
    return r2R2MarkerArray[1];
}

function getr2R2Marker95()
{
    return r2R2MarkerArray[2];
}

function getr2Marker50()
{
    return r2MarkerArray[0];
}

function getr2Marker90()
{
    return r2MarkerArray[1];
}

function getr2Marker95()
{
    return r2MarkerArray[2];
}

function getrMarker50()
{
    return rMarkerArray[0];
}

function getrMarker90()
{
    return rMarkerArray[1];
}

function getrMarker95()
{
    return rMarkerArray[2];
}

function getSchrodingerData(n, l)
// PRE:  n and l must be valid quantum numbers
// POST: FCTVAL = psi(r) sub n,l. TWO arrays FCTVAL[0] = positive and FCTVAL[1] = negative
{
    /* error in input */
    if (n == null || l == null)
        return;
    
    
    var a0 = 0.529 * Math.pow(10, -10);         // bohr radius in meters
    var bohrRadius = 0.529;                     // bohr radius in angstroms
    var numer;                                  // numerator
    var denom;                                  // denominator
    var sumResult;                              // result of the summation
    var constant;                               // constant part of psi(r) equation for n and l
    var radius;                                 // the current radius (in meters)
    var radiusInAngstroms;                      // the current radius in angstroms
    var r2p2max;                                // max value of psi^2 * 4pir^2
    var p2max;                                  // max value of psi^2
    var pmax;                                   // max value of psi
    var psi;                                    // value of psi(r)
    var psiSquared;                             // psi^2
    var rSquaredPsiSquared;                     // psi^2 * 4pir^2
    var numValues;                              // number of values to graph
    var extent;                                 // x axis extent of the schrodiger graphs    
    var marker50Radius;                         // radius of the 50% marker
    var marker90Radius;                         // radius of the 90% marker
    var marker95Radius;                         // radius of the 95% marker
    var schAvg;
    var totalValues;                            // sum of all values
    var markerPadding;                          // the "padding" on each side of a marker so the marker can actually be clicked
    var markerRange;                            // how close a value has to be to the marker radius in order for it to be consider the correct radius
    var r2R2SingleArray;                         // the r2R2 values in a nonsplit array
    var mostProbableRadius;
    var r2SingleArray;
    /*
    if (stepSize == null)
    {
        stepSize = CONST_STEP_SIZE;
    }
    */
    
    stepSize = CONST_STEP_SIZE;
    
    markerRange = stepSize * Math.pow(10, -10) / 2;
    schrodingerMostProbRadius = 0;
    markerPadding = 8;
    extent = getFunctionExtent(n, l);
    numValues = extent / stepSize;
    schAvg = 0;
    totalValues = 0;
    // the same for all values of psi(r) for constant r and l. no need to recalculate.
    constant = Math.pow( (2/(n*a0)), 1.5 ) * Math.pow(  ( (factorial(n-l-1) * factorial(n+l)) / (2*n) ), .5 );
    // get the radii for the markers
    marker50Radius = ORBITAL_PROBABILITY_50[n-1][l] * bohrRadius;    
    marker90Radius = ORBITAL_PROBABILITY_90[n-1][l] * bohrRadius;
    marker95Radius = ORBITAL_PROBABILITY_95[n-1][l] * bohrRadius;
    
    // initalize the arrays
    negr2p2Data = new Array;
    posr2p2Data = new Array;
    posp2Data = new Array;
    negp2Data = new Array;
    posPsiData = new Array;
    negPsiData = new Array;
    r2R2MarkerArray = new Array;
    r2MarkerArray = new Array;
    rMarkerArray = new Array;
    r2R2SingleArray = new Array;
    r2SingleArray = new Array;
    rSingleArray = new Array;
    
    // numValues values at stepSize intervals
    for(var i = 0; i < numValues; i++)
    {
        //the radius at which to evaluate
        radiusInAngstroms = stepSize * i;
        radius = roundNumber(radiusInAngstroms * Math.pow(10, -10), 12) 
        
        // the summation
        sumResult = 0;
        for(var j = 0; j <= n - l - 1; j++)
        {
            numer = Math.pow( -2*radius/(n*a0), j);
            denom = factorial(j)*factorial(n-l-j-1)*factorial(2*l+j+1);
            sumResult += (numer/denom);
        }

        // product of everything
        psi = constant * Math.pow( 2*radius/(n*a0), l ) * Math.pow( Math.E, -1*radius/(n*a0) ) * sumResult;

        psiSquared = Math.pow(psi, 2);
        
        rSquaredPsiSquared = psiSquared * 4 * Math.PI * Math.pow(radius, 2);
                         
        schAvg += rSquaredPsiSquared * radiusInAngstroms;
        totalValues += rSquaredPsiSquared;

        
        r2R2SingleArray.push( [radius * Math.pow(10, 10), rSquaredPsiSquared] );
        r2SingleArray.push( [radius * Math.pow(10, 10), psiSquared] );
        rSingleArray.push( [radius * Math.pow(10, 10), psi] );
                
        if ( !(Math.abs(radiusInAngstroms - marker50Radius) <= markerPadding * stepSize ||
               Math.abs(radiusInAngstroms - marker90Radius) <= markerPadding * stepSize ||
               Math.abs(radiusInAngstroms - marker95Radius) <= markerPadding * stepSize ))
        {
            if(psi >= 0)                                                                // Positive
            {
                posr2p2Data.push( [radius * Math.pow(10, 10), rSquaredPsiSquared] );
                posp2Data.push( [radius * Math.pow(10, 10), psiSquared] );
                posPsiData.push( [radius * Math.pow(10, 10), psi] );
            }
            else                                                                         // if (psi < 0)
            {
                negr2p2Data.push( [radius * Math.pow(10, 10), rSquaredPsiSquared] );
                negp2Data.push( [radius * Math.pow(10, 10), psiSquared] );
                negPsiData.push( [radius * Math.pow(10, 10), psi] );
            }
        }     

        if(i == 0 || r2p2max < rSquaredPsiSquared)                                  // check for maxes
        {
            r2p2max = rSquaredPsiSquared;
            mostProbableRadius = radius * Math.pow(10, 10);
        }

        if(i == 0 || p2max < psiSquared)                                            // check for maxes
            p2max = psiSquared;

        if(i == 0 || pmax < Math.abs(psi))                                                    // check for maxes
            pmax = Math.abs(psi);


    }

    // normalize all data
    for(var i = 0; i < posr2p2Data.length; i++)
    {
        posr2p2Data[i][1] = posr2p2Data[i][1] / r2p2max;
        posp2Data[i][1] = posp2Data[i][1] / p2max;
        posPsiData[i][1] = posPsiData[i][1] / pmax;
    }
    for(var i = 0; i < negr2p2Data.length; i++)
    {
        negp2Data[i][1] = negp2Data[i][1] / p2max;
        negPsiData[i][1] = negPsiData[i][1] / pmax;
        negr2p2Data[i][1] = negr2p2Data[i][1] / r2p2max;
    }
    
    for(var i = 0; i < r2R2SingleArray.length; i++)
    {
        r2R2SingleArray[i][1] = r2R2SingleArray[i][1] / r2p2max;
        r2SingleArray[i][1] = r2SingleArray[i][1] / p2max
        rSingleArray[i][1] = rSingleArray[i][1] / pmax
    }
    
    var tempArray = new Array;
    
    for(var i = 1; i < rSingleArray.length; i++)
    {
        if ( rSingleArray[i-1][1] * rSingleArray[i][1] <= 0)
        {
            tempArray.push(rSingleArray[i][0]);
        }
    }
    //alert(tempArray);
                
    r2R2MarkerArray.push( r2R2SingleArray[ Math.floor( marker50Radius / stepSize ) ] );
    r2R2MarkerArray.push( r2R2SingleArray[ Math.floor( marker90Radius / stepSize ) ] );
    r2R2MarkerArray.push( r2R2SingleArray[ Math.floor( marker95Radius / stepSize ) ] );
    
    r2MarkerArray.push( r2SingleArray[ Math.floor( marker50Radius / stepSize ) ] );
    r2MarkerArray.push( r2SingleArray[ Math.floor( marker90Radius / stepSize ) ] );
    r2MarkerArray.push( r2SingleArray[ Math.floor( marker95Radius / stepSize ) ] );
    
    rMarkerArray.push( rSingleArray[ Math.floor( marker50Radius / stepSize ) ] );
    rMarkerArray.push( rSingleArray[ Math.floor( marker90Radius / stepSize ) ] );
    rMarkerArray.push( rSingleArray[ Math.floor( marker95Radius / stepSize ) ] );
    
    schrodingerMostProbRadius = mostProbableRadius;
    schrodingerAvgRadius = schAvg / totalValues;
    //schrodingerAvgRadius = Math.pow(schrodingerN, 2) * ( 3/2 - (schrodingerL * (schrodingerL + 1)) / (2 * Math.pow(schrodingerN, 2)));
}

function factorial(number)
// PRE:  number >= 0
// POST: FCTVAL = number!
{
    var factorialResult;                    // the factorial

    if(number > 0)                       // NOT factorial of 0
    {
        factorialResult = number;

        for(var i = number - 1; i > 0; i--)
           factorialResult = factorialResult * i;
    }
    else                                   // factorial of 0
        factorialResult = 1;

    return factorialResult;
}

function roundNumber(number, numDecimals)
// PRE: number must be defined, numDecimals > 0
{
    return Math.round(number * Math.pow(10, numDecimals)) / Math.pow(10, numDecimals);
}


function getFunctionExtent(n, l)
// PRE:  n and l must be valid quantum numbers where n is less than MAX_N_MAGNITUDE  NOT getting local extent var!!
// POST: FCTVAL = extent
{
    return ((n > 0 && n <= MAX_N_MAGNITUDE && l >= 0 && l < n ) ? ORBITAL_EXTENTS[n-1][l] : 0);
}